from __future__ import annotations

from contextlib import AbstractContextManager
from dataclasses import dataclass
from time import perf_counter

try:
    from playwright.sync_api import Error as PlaywrightError
    from playwright.sync_api import sync_playwright
except ImportError:  # pragma: no cover - runtime dependency
    sync_playwright = None
    PlaywrightError = Exception


@dataclass
class PageFetch:
    response: object | None
    elapsed_ms: float
    resources: list[dict]


class Browser(AbstractContextManager):
    def __init__(self, *, headless: bool, timeout_ms: int, wait_until: str) -> None:
        self.headless = headless
        self.timeout_ms = timeout_ms
        self.wait_until = wait_until
        self._playwright = None
        self._browser = None
        self._context = None

    def __enter__(self) -> "Browser":
        if sync_playwright is None:
            raise RuntimeError(
                "Playwright is not installed. Run: uv pip install playwright && playwright install"
            )
        self._playwright = sync_playwright().start()
        self._browser = self._playwright.chromium.launch(headless=self.headless)
        self._context = self._browser.new_context()
        return self

    def __exit__(self, exc_type, exc, tb) -> None:
        if self._context is not None:
            self._context.close()
        if self._browser is not None:
            self._browser.close()
        if self._playwright is not None:
            self._playwright.stop()

    def open(self, url: str) -> tuple[object | None, object | None, float, list[dict]]:
        if self._context is None:
            raise RuntimeError("Browser not initialized")
        page = self._context.new_page()
        resources: list[dict] = []

        def handle_response(resp) -> None:
            try:
                request = resp.request
                resources.append(
                    {
                        "url": resp.url,
                        "status": resp.status,
                        "resource_type": request.resource_type,
                    }
                )
            except Exception:
                return

        page.on("response", handle_response)
        start = perf_counter()
        response = None
        try:
            response = page.goto(url, wait_until=self.wait_until, timeout=self.timeout_ms)
        except PlaywrightError:
            pass
        elapsed_ms = (perf_counter() - start) * 1000
        return page, response, elapsed_ms, resources



"""WORAi root CLI."""

from __future__ import annotations

import logging

import typer

from worai.config import load_config
from worai.errors import WoraError
from worai.logging import setup_logging

app = typer.Typer(add_completion=True, no_args_is_help=True)


@app.callback()
def main(
    ctx: typer.Context,
    config: str | None = typer.Option(None, "--config", help="Path to config TOML."),
    profile: str | None = typer.Option(None, "--profile", help="Config profile name."),
    log_level: str = typer.Option("info", "--log-level", help="Log level."),
    log_format: str = typer.Option("text", "--log-format", help="Log format: text or json."),
    quiet: bool = typer.Option(False, "--quiet", help="Only warnings and errors."),
) -> None:
    """WORAi command-line tools."""
    setup_logging(level=log_level, fmt=log_format, quiet=quiet)
    ctx.obj = {
        "config": load_config(config, profile),
        "log_level": log_level,
        "log_format": log_format,
        "quiet": quiet,
    }


@app.command("version")
def version() -> None:
    """Print version."""
    from worai import __version__

    typer.echo(__version__)


def _install_commands() -> None:
    from worai.commands import (
        canonicalize_duplicate_pages,
        delete_entities_from_csv,
        dedupe,
        find_faq_page_wrong_type,
        find_missing_names,
        find_url_by_type,
        google_search_console,
        link_groups,
        patch,
        seocheck,
        upload_entities_from_turtle,
    )

    app.add_typer(seocheck.app, name="seocheck")
    app.add_typer(google_search_console.app, name="google-search-console")
    app.add_typer(dedupe.app, name="dedupe")
    app.add_typer(canonicalize_duplicate_pages.app, name="canonicalize-duplicate-pages")
    app.add_typer(delete_entities_from_csv.app, name="delete-entities-from-csv")
    app.add_typer(find_faq_page_wrong_type.app, name="find-faq-page-wrong-type")
    app.add_typer(find_missing_names.app, name="find-missing-names")
    app.add_typer(find_url_by_type.app, name="find-url-by-type")
    app.add_typer(link_groups.app, name="link-groups")
    app.add_typer(patch.app, name="patch")
    app.add_typer(upload_entities_from_turtle.app, name="upload-entities-from-turtle")


_install_commands()


def run() -> None:
    try:
        app()
    except WoraError as exc:
        logging.getLogger("worai").error(str(exc))
        raise SystemExit(exc.exit_code) from exc
